from django.test import TestCase, Client
from django.db.utils import IntegrityError
from .models import Lokasi, Fasilitas, Komentar, KURSI_RODA, RUNNING_TEXT
from .serializers import LokasiSerializer
from registrasi.models import BisaGoUser
from django.contrib.auth.models import User
from django.urls import reverse, path, include
import json
import tempfile

# Create your tests here.
class InformasiFasilitasModelTest(TestCase):
    not_null_constraint_failed_message = 'NOT NULL constraint failed'

    def test_models_lokasi_not_created(self):
        with self.assertRaises(IntegrityError) as cm:
            obj = Lokasi(name=None)
            obj.save()
        self.assertTrue(str(cm.exception).startswith(
            self.not_null_constraint_failed_message))

    def test_models_create_new_lokasi(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        Lokasi.objects.create(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        count = Lokasi.objects.all().count()
        self.assertNotEqual(count, 0)

    def test_models_fasilitas_not_created(self):
        with self.assertRaises(IntegrityError) as cm:
            obj = Fasilitas(lokasi=None)
            obj.save()
        self.assertTrue(str(cm.exception).startswith(
            self.not_null_constraint_failed_message))
        
    def test_models_create_new_fasilitas(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        user=User(
            username='user@gmail.com',
            last_name='name',
            email='user@gmail.com', 
            password='hahagotim'
        )
        user.save()
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        Fasilitas.objects.create(
            lokasi=lokasi, 
            user=user, 
            deskripsi="penjelasan panjang", 
            like=0, 
            dislike=0, 
            rating=5, 
            tag={KURSI_RODA, RUNNING_TEXT}, 
            image=image
        )
        count = Fasilitas.objects.all().count()
        self.assertNotEqual(count, 0)

    def test_models_komentar_not_created(self):
        with self.assertRaises(IntegrityError) as cm:
            obj = Komentar(fasilitas=None)
            obj.save();
        self.assertTrue(str(cm.exception).startswith(
            self.not_null_constraint_failed_message))

    def test_models_create_new_komentar(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        user=User(
            username='user',
            last_name='name',
            email='user@gmail.com', 
            password='hahagotim'
        )
        user.save()
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        fasilitas=Fasilitas(
            lokasi=lokasi, 
            user=user, 
            deskripsi="penjelasan panjang", 
            like=0, 
            dislike=0, 
            rating=5, 
            tag={KURSI_RODA, RUNNING_TEXT}, 
            image=image
        )
        fasilitas.save()
        Komentar.objects.create(
            user=user,
            fasilitas=fasilitas,
            deskripsi="penjelasan panjang"
        )
        count = Komentar.objects.all().count()
        self.assertNotEqual(count, 0)

def test_view_lokasi(self):
    image = tempfile.NamedTemporaryFile(suffix=".jpg").name
    lokasi=Lokasi(
        name='Sekolah',
        latitude=0.0,
        longitude=0.0,
        alamat='Soppeng sentosa',
        no_telp='081212123131',
        image=image
    )
    lokasi.save()

class InformasiFasilitasViewTest(TestCase):
    urlpatterns = [
        path('informasi-fasilitas/', include('informasi_fasilitas.urls')),
    ]

    def test_LokasiSerializer_valid(self):
        serializer = LokasiSerializer(data = {
            'name' : 'Mall1',
            'latitude' : 0.0,
            'longitude' : 0.0,
            'alamat' : 'Jl. Raya Bogor no.1, Jakarta',
            'no_telp' : '081212123131'
            })
        self.assertTrue(serializer.is_valid)

    def test_can_get_lokasi_list(self):
        response = Client().get(reverse('lokasi-list'))
        self.assertEqual(response.status_code, 200)

    def test_cannot_post_lokasi_list(self):
        response = Client().post(reverse('lokasi-list'))
        self.assertEqual(response.status_code, 405)

    def test_can_get_lokasi_details(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        response = Client().get(reverse('lokasi-details', kwargs={'name':'Mall'}))
        self.assertEqual(response.status_code, 200)

    def test_cannot_post_lokasi_details(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        response = Client().post(reverse('lokasi-details', kwargs={'name':'Mall'}))
        self.assertEqual(response.status_code, 405)

    def test_get_lokasi_details_not_found(self):
        response = Client().get(reverse('lokasi-details', kwargs={'name':'Mall'}))
        self.assertEqual(response.status_code, 404)

    def test_can_post_add_lokasi(self):
        email='user@gmail.com'
        password='hahagotim'
        Client().post('/api/register/',
        {'name':'name', 
        'email':email,
        'phone_number':000000000, 
        'password':password})
        token_response = Client().post('/api-token-auth/',
        {'username' : email, 'password' : password})
        content = json.loads(token_response.content.decode('utf-8'))
        token = content['token']
        client = Client(HTTP_AUTHORIZATION='token '+token)

        response = client.post(reverse('add-lokasi'), 
        {'name' : 'Mall',
        'latitude' : 0.0,
        'longitude' : 0.0,
        'alamat' : 'Jl. Raya Bogor no.1, Jakarta',
        'no_telp' : '081212123131'})
        self.assertEqual(response.status_code, 201)
    
    def test_cannot_get_add_lokasi(self):
        email='user@gmail.com'
        password='hahagotim'
        Client().post('/api/register/',
        {'name':'name', 
        'email':email,
        'phone_number':000000000, 
        'password':password})
        token_response = Client().post('/api-token-auth/',
        {'username' : email, 'password' : password})
        content = json.loads(token_response.content.decode('utf-8'))
        token = content['token']
        client = Client(HTTP_AUTHORIZATION='token '+token)

        response = client.get(reverse('add-lokasi'))
        self.assertEqual(response.status_code, 405)

    def test_can_get_list_fasilitas(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        user=User(
            username='user@gmail.com',
            last_name='name',
            email='user@gmail.com', 
            password='hahagotim'
        )
        user.save()
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        fasilitas=Fasilitas(
            lokasi=lokasi, 
            user=user, 
            deskripsi="penjelasan panjang", 
            like=0, 
            dislike=0, 
            rating=5, 
            tag={KURSI_RODA, RUNNING_TEXT}, 
            image=image
        )
        fasilitas.save()
        response = Client().get(reverse('list-fasilitas', kwargs={'nama_lokasi':'Mall'}))
        self.assertEqual(response.status_code, 200)

    def test_cannot_post_list_fasilitas(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        response = Client().post(reverse('list-fasilitas', kwargs={'nama_lokasi':'Mall'}))
        self.assertEqual(response.status_code, 405)

    def test_get_list_fasilitas_not_found(self):
        response = Client().get(reverse('list-fasilitas', kwargs={'nama_lokasi':'Mall'}))
        self.assertEqual(response.status_code, 404)

    def test_can_get_detail_fasilitas(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        user=User(
            username='user@gmail.com',
            last_name='name',
            email='user@gmail.com', 
            password='hahagotim'
        )
        user.save()
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        fasilitas=Fasilitas(
            lokasi=lokasi, 
            user=user, 
            deskripsi="penjelasan panjang", 
            like=0, 
            dislike=0, 
            rating=5, 
            tag={KURSI_RODA, RUNNING_TEXT}, 
            image=image
        )
        fasilitas.save()
        id = fasilitas.id
        response = Client().get(reverse('detail-fasilitas', kwargs={'nama_lokasi':'Mall', 'id':id}))
        self.assertEqual(response.status_code, 200)

    def test_cannot_post_detail_fasilitas(self):
        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        user=User(
            username='user@gmail.com',
            last_name='name',
            email='user@gmail.com', 
            password='hahagotim'
        )
        user.save()
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()
        fasilitas=Fasilitas(
            lokasi=lokasi, 
            user=user, 
            deskripsi="penjelasan panjang", 
            like=0, 
            dislike=0, 
            rating=5, 
            tag={KURSI_RODA, RUNNING_TEXT}, 
            image=image
        )
        fasilitas.save()
        id = fasilitas.id
        response = Client().post(reverse('detail-fasilitas', kwargs={'nama_lokasi':'Mall', 'id':id}))
        self.assertEqual(response.status_code, 405)

    def test_get_detail_fasilitas_not_found(self):
        response = Client().get(reverse('detail-fasilitas', kwargs={'nama_lokasi':'Mall', 'id':0}))
        self.assertEqual(response.status_code, 404)

    def test_can_post_add_fasilitas(self):
        email='user@gmail.com'
        password='hahagotim'
        Client().post('/api/register/',
        {'name':'name', 
        'email':email,
        'phone_number':000000000, 
        'password':password})
        token_response = Client().post('/api-token-auth/',
        {'username' : email, 'password' : password})
        content = json.loads(token_response.content.decode('utf-8'))
        token = content['token']
        client = Client(HTTP_AUTHORIZATION='token '+token)

        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()

        response = client.post(reverse('add-fasilitas', kwargs={'nama_lokasi':'Mall'}), 
        {'deskripsi' : 'penjelasan fasilitas',
        'rating' : 2,
        'tag' : 'KR RT'})
        self.assertEqual(response.status_code, 201)
    
    def test_cannot_get_add_fasilitas(self):
        email='user@gmail.com'
        password='hahagotim'
        Client().post('/api/register/',
        {'name':'name', 
        'email':email,
        'phone_number':000000000, 
        'password':password})
        token_response = Client().post('/api-token-auth/',
        {'username' : email, 'password' : password})
        content = json.loads(token_response.content.decode('utf-8'))
        token = content['token']
        client = Client(HTTP_AUTHORIZATION='token '+token)

        image = tempfile.NamedTemporaryFile(suffix=".jpg").name
        lokasi=Lokasi(
            name='Mall', 
            latitude=0.0, 
            longitude=0.0, 
            alamat='Jl. Raya Bogor no.1, Jakarta', 
            no_telp='081212123131',
            image=image
        )
        lokasi.save()

        response = client.get(reverse('add-fasilitas', kwargs={'nama_lokasi':'Mall'}))
        self.assertEqual(response.status_code, 405)

    def test_post_add_fasilitas_not_found(self):
        email='user@gmail.com'
        password='hahagotim'
        Client().post('/api/register/',
        {'name':'name', 
        'email':email,
        'phone_number':000000000, 
        'password':password})
        token_response = Client().post('/api-token-auth/',
        {'username' : email, 'password' : password})
        content = json.loads(token_response.content.decode('utf-8'))
        token = content['token']
        client = Client(HTTP_AUTHORIZATION='token '+token)

        response = client.post(reverse('add-fasilitas', kwargs={'nama_lokasi':'Mall'}), 
        {'deskripsi' : 'penjelasan fasilitas',
        'rating' : 2})
        self.assertEqual(response.status_code, 404)